@@ -55,6 +55,7 @@
 #include <sys_defs.h>
 #include <stdlib.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
@@ -103,9 +104,71 @@
 #include <smtp_addr.h>
 #include <smtp_reuse.h>
 
+ /*
+  * Forward declaration.
+  */
+static SMTP_SESSION *smtp_connect_sock(int, struct sockaddr *, int,
+				               const char *, const char *,
+				               unsigned,
+				               const char *, DSN_BUF *,
+				               int);
+
+/* smtp_connect_unix - connect to UNIX-domain address */
+
+static SMTP_SESSION *smtp_connect_unix(const char *addr,
+				               const char *destination,
+				               DSN_BUF *why,
+				               int sess_flags)
+{
+#undef sun
+    char   *myname = "smtp_connect_unix";
+    struct sockaddr_un sun;
+    int     len = strlen(addr);
+    int     sock;
+
+    smtp_errno = SMTP_ERR_NONE;			/* Paranoia */
+
+    /*
+     * Sanity checks.
+     */
+    if (len >= (int) sizeof(sun.sun_path)) {
+	msg_warn("unix-domain name too long: %s", addr);
+	smtp_dsn_update(why, DSN_BY_LOCAL_MTA, "4.3.5",
+			450, "450 Mail server configuration error",
+			"Server configuration error");
+	smtp_errno = SMTP_ERR_RETRY;
+	return (0);
+    }
+
+    /*
+     * Initialize.
+     */
+    memset((char *) &sun, 0, sizeof(sun));
+    sun.sun_family = AF_UNIX;
+#ifdef HAS_SUN_LEN
+    sun.sun_len = len + 1;
+#endif
+    memcpy(sun.sun_path, addr, len + 1);
+
+    /*
+     * Create a client socket.
+     */
+    if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
+	msg_fatal("%s: socket: %m", myname);
+
+    /*
+     * Connect to the server.
+     */
+    if (msg_verbose)
+	msg_info("%s: trying: %s...", myname, addr);
+
+    return (smtp_connect_sock(sock, (struct sockaddr *) & sun, sizeof(sun),
+			      addr, addr, 0, destination, why, sess_flags));
+}
+
 /* smtp_connect_addr - connect to explicit address */
 
-static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
+static SMTP_SESSION *smtp_connect_addr(const char *destination, DNS_RR *addr,
 				               unsigned port, DSN_BUF *why,
 				               int sess_flags)
 {
@@ -115,13 +178,8 @@ static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
     SOCKADDR_SIZE salen = sizeof(ss);
     MAI_HOSTADDR_STR hostaddr;
     int     sock;
-    int     conn_stat;
-    int     saved_errno;
-    VSTREAM *stream;
-    int     ch;
     char   *bind_addr;
     char   *bind_var;
-    time_t  start_time;
 
     smtp_errno = SMTP_ERR_NONE;			/* Paranoia */
 
@@ -207,12 +265,33 @@ static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
     }
 
     /*
-     * Connect to the SMTP server.
+     * Connect to the server.
      */
     SOCKADDR_TO_HOSTADDR(sa, salen, &hostaddr, (MAI_SERVPORT_STR *) 0, 0);
     if (msg_verbose)
 	msg_info("%s: trying: %s[%s] port %d...",
 		 myname, addr->name, hostaddr.buf, ntohs(port));
+
+    return (smtp_connect_sock(sock, sa, salen, addr->name, hostaddr.buf,
+			      port, destination, why, sess_flags));
+}
+
+/* smtp_connect_sock - connect a socket over some transport */
+
+static SMTP_SESSION *smtp_connect_sock(int sock, struct sockaddr * sa,
+				               int salen, const char *name,
+				               const char *addr,
+				               unsigned port,
+				               const char *destination,
+				               DSN_BUF *why,
+				               int sess_flags)
+{
+    int     conn_stat;
+    int     saved_errno;
+    VSTREAM *stream;
+    int     ch;
+    time_t  start_time;
+
     start_time = time((time_t *) 0);
     if (var_smtp_conn_tmout > 0) {
 	non_blocking(sock, NON_BLOCKING);
@@ -227,12 +306,20 @@ static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
     if (conn_stat < 0) {
 	smtp_dsn_update(why, DSN_BY_LOCAL_MTA,
 			"4.4.1", 420, "420 Unable to connect to server",
-			"connect to %s[%s]: %m", addr->name, hostaddr.buf);
+			"connect to %s[%s]: %m", name, addr);
 	smtp_errno = SMTP_ERR_RETRY;
 	close(sock);
 	return (0);
     }
 
+    /*
+     * Following code is obsolete now that the SMTP client will connect to
+     * alternate hosts when a session fails before "MAIL FROM".
+     */
+#if 1
+    stream = vstream_fdopen(sock, O_RDWR);
+#else
+
     /*
      * Skip this host if it takes no action within some time limit. XXX Some
      * MTAs use 426 to indicate a timeout error.
@@ -261,11 +348,12 @@ static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
 	return (0);
     }
     vstream_ungetc(stream, ch);
+#endif
 
     /*
      * Bundle up what we have into a nice SMTP_SESSION object.
      */
-    return (smtp_session_alloc(stream, dest, addr->name, hostaddr.buf,
+    return (smtp_session_alloc(stream, destination, name, addr,
 			       port, start_time, sess_flags));
 }
 
@@ -274,7 +362,7 @@ static SMTP_SESSION *smtp_connect_addr(const char *dest, DNS_RR *addr,
 static char *smtp_parse_destination(char *destination, char *def_service,
 				            char **hostp, unsigned *portp)
 {
-    char   *buf = mystrdup(destination);
+    char   *buf;
     char   *service;
     struct servent *sp;
     char   *protocol = "tcp";		/* XXX configurable? */
@@ -284,6 +372,27 @@ static char *smtp_parse_destination(char *destination, char *def_service,
     if (msg_verbose)
 	msg_info("smtp_parse_destination: %s %s", destination, def_service);
 
+    /*
+     * XXX Support for UNIX-domain destinations requires that we split the
+     * smtp_connect() function into the portions that are dedicated to TCP,
+     * and the portions that can be shared with TCP and UNIX-domain sockets.
+     */
+    if (strncmp(destination, "unix:", 5) == 0) {
+	msg_fatal("UNIX-domain is currently unimplemented");
+	*hostp = buf = mystrdup(destination + 5);
+	*portp = 0;
+	return (buf);
+    }
+
+    /*
+     * XXX And while we're fixing that, we should also ensure that the inet:
+     * prefix is recognized with LMTP destinations only.
+     */
+    if (strncmp(destination, "inet:", 5) == 0)
+	buf = mystrdup(destination + 5);
+    else
+	buf = mystrdup(destination);
+
     /*
      * Parse the host/port information. We're working with a copy of the
      * destination argument so the parsing can be destructive.
@@ -295,7 +404,7 @@ static char *smtp_parse_destination(char *destination, char *def_service,
      * Convert service to port number, network byte order.
      */
     if (alldig(service)) {
-	if ((port = atoi(service)) >= 65536)
+	if ((port = atoi(service)) >= 65536 || port == 0)
 	    msg_fatal("bad network port in destination: %s", destination);
 	*portp = htons(port);
     } else {
@@ -310,6 +419,7 @@ static char *smtp_parse_destination(char *destination, char *def_service,
 
 static void smtp_cleanup_session(SMTP_STATE *state)
 {
+    DELIVER_REQUEST *request = state->request;
     SMTP_SESSION *session = state->session;
     int     bad_session;
 
@@ -357,6 +467,27 @@ static void smtp_cleanup_session(SMTP_STATE *state)
      * Clean up the lists with todo and dropped recipients.
      */
     smtp_rcpt_cleanup(state);
+
+    /*
+     * Reset profiling info.
+     * 
+     * XXX When one delivery request results in multiple sessions, the set-up
+     * and transmission latencies of the earlier sessions will count as
+     * connection set-up time for the later sessions.
+     * 
+     * XXX On the other hand, when we first try to connect to one or more dead
+     * hosts before we reach a good host, then all that time must be counted
+     * as connection set-up time for the session with the good host.
+     * 
+     * XXX So this set-up attribution problem exists only when we actually
+     * engage in a session, spend a lot of time delivering a message, find
+     * that it fails, and then connect to an alternate host.
+     */
+    memset((char *) &request->msg_stats.conn_setup_done, 0,
+	   sizeof(request->msg_stats.conn_setup_done));
+    memset((char *) &request->msg_stats.deliver_done, 0,
+	   sizeof(request->msg_stats.deliver_done));
+    request->msg_stats.reuse_count = 0;
 }
 
 /* smtp_scrub_address_list - delete all cached addresses from list */
@@ -495,7 +626,7 @@ int     smtp_connect(SMTP_STATE *state)
     char   *dest_buf;
     char   *domain;
     unsigned port;
-    char   *def_service = "smtp";	/* XXX ##IPPORT_SMTP? */
+    char   *def_service;
     ARGV   *sites;
     char   *dest;
     char  **cpp;
@@ -504,7 +635,6 @@ int     smtp_connect(SMTP_STATE *state)
     DNS_RR *next;
     int     addr_count;
     int     sess_count;
-    int     misc_flags = SMTP_MISC_FLAG_DEFAULT;
     SMTP_SESSION *session;
     int     lookup_mx;
     unsigned domain_best_pref;
@@ -512,6 +642,9 @@ int     smtp_connect(SMTP_STATE *state)
     int     i_am_mx = 0;
     int     non_fallback_sites;
 
+    def_service = (state->misc_flags & SMTP_MISC_FLAG_USE_LMTP) ?
+	var_lmtp_tcp_port : "smtp";		/* XXX ##IPPORT_SMTP? */
+
     /*
      * First try to deliver to the indicated destination, then try to deliver
      * to the optional fall-back relays.
@@ -561,15 +694,17 @@ int     smtp_connect(SMTP_STATE *state)
 	if (msg_verbose)
 	    msg_info("connecting to %s port %d", domain, ntohs(port));
 	if (ntohs(port) != IPPORT_SMTP)
-	    misc_flags &= ~SMTP_MISC_FLAG_LOOP_DETECT;
+	    state->misc_flags &= ~SMTP_MISC_FLAG_LOOP_DETECT;
 	else
-	    misc_flags |= SMTP_MISC_FLAG_LOOP_DETECT;
-	lookup_mx = (var_disable_dns == 0 && *dest != '[');
+	    state->misc_flags |= SMTP_MISC_FLAG_LOOP_DETECT;
+	lookup_mx = (var_disable_dns == 0 && *dest != '['
+		     && (state->misc_flags & SMTP_MISC_FLAG_USE_LMTP) == 0);
 	if (!lookup_mx) {
-	    addr_list = smtp_host_addr(domain, misc_flags, why);
+	    addr_list = smtp_host_addr(domain, state->misc_flags, why);
 	    /* XXX We could be an MX host for this destination... */
 	} else {
-	    addr_list = smtp_domain_addr(domain, misc_flags, why, &i_am_mx);
+	    addr_list = smtp_domain_addr(domain, state->misc_flags,
+					 why, &i_am_mx);
 	    /* If we're MX host, don't connect to non-MX backups. */
 	    if (i_am_mx)
 		argv_truncate(sites, cpp - sites->argv + 1);
@@ -594,8 +729,14 @@ int     smtp_connect(SMTP_STATE *state)
 	 * Opportunistic (a.k.a. on-demand) session caching on request by the
 	 * queue manager. This is turned temporarily when a destination has a
 	 * high volume of mail in the active queue.
+	 * 
+	 * XXX Disable connection caching when sender-dependent authentication
+	 * is enabled. We must not send someone elses mail over an
+	 * authenticated connection, and we must not send mail that requires
+	 * authentication over a connection that wasn't authenticated.
 	 */
 	if (cpp == sites->argv
+	    && !var_smtp_sender_auth
 	    && ((var_smtp_cache_demand && (request->flags & DEL_REQ_FLAG_SCACHE) != 0)
 		|| (smtp_cache_dest && string_list_match(smtp_cache_dest, domain)))) {
 	    sess_flags |= SMTP_SESS_FLAG_CACHE;
@@ -662,12 +803,13 @@ int     smtp_connect(SMTP_STATE *state)
 		|| (session = smtp_reuse_addr(state, addr, port)) == 0)
 		session = smtp_connect_addr(dest, addr, port, why, sess_flags);
 	    if ((state->session = session) != 0) {
+		session->state = state;
 		if (addr->pref == domain_best_pref)
 		    session->features |= SMTP_FEATURE_BEST_MX;
 		/* Don't count handshake errors towards the session limit. */
 		state->final_server = (cpp[1] == 0 && next == 0);
 		if ((session->features & SMTP_FEATURE_FROM_CACHE) == 0
-		    && smtp_helo(state, misc_flags) != 0) {
+		    && smtp_helo(state) != 0) {
 		    if (vstream_ferror(session->stream) == 0
 			&& vstream_feof(session->stream) == 0)
 			smtp_quit(state);
